<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Db_Table_Abstract
 */
require_once 'Zend/Db/Table/Abstract.php';

/**
 * Abstract class for table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Db_Table_Abstract extends Zend_Db_Table_Abstract
{

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Constructor
     *
     * @param array $options Available options
     * @return void
     */
    public function __construct ($options = array())
    {
        
        $config = Zend_Registry::get('config');
        
        if (empty($this->_schema)) {
            $this->_schema = $config->database->schema;
        }
        
        parent::__construct($options);
    }

    /**
     * Fetch rows that belong to current user
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereCurrentUser bool - Fetch the row that belongs to current user</li></ul>
     * @return void
     */
    public function whereCurrentUser ($select, array $options = array())
    {
        if (empty($options['whereCurrentUser'])) {
            return;
        }
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $select->where("{$this->_name}.user_id = ?", $userId);
    
    }

    /**
     * Fetch active or inactive users
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereActive t,f - If value is 't' fetches active users, 'f' inactive</li></ul>
     * @return void
     */
    public function whereActive ($select, array $options = array())
    {
        if (empty($options['whereActive'])) {
            return;
        }
            
        $select->where($this->_name . '.active = ?', $options['whereActive']);
    }

    /**
     * Fetch id
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereId integer - Fetch the row with a given id</li></ul>
     * @return void
     */
    public function whereId ($select, array $options = array())
    {
        if (! isset($options['whereId'])) {
            return;
        }
        
        $select->where($this->_name . '.id = ?', $options['whereId']);
    }
        
    /**
     * Add multilanguage fields based on current language
     * 
     * @param Zend_Db_Select $select
     * @param string $name - language abbreviation
     * @param string $table - table to join
     * @param string $schema - database schema
     * @return void
     */
    public function joinMultilangField ($select, $name, $table = null, $schema = null)
    {
        $lang = Zend_Registry::get('Zend_Translate')->getLocale();
        
        if(empty($table)) {
            $table = $this->_name;
        }
        
        if(empty($schema)) {
            $schema = $this->_schema;
        }
        
        $select->joinLeft(array(
            'text_' . $name => $schema . '.text'
        ), "text_{$name}.id = {$table}.{$name}_id", array(
            $name => $lang
        ));
    }
    
    /**
     * Count the number of items in the table
     * 
     * @param array $options Available options:
     * <ul><li>onlyActive bool - Counts only active rows</li></ul>
     * @return integer
     */
    public function count ($options = array())
    {
        $select = $this->select();
        $select->from($this, array(
            'count' => 'count(*)'
        ));
        
        if (! empty($options['onlyActive'])) {
            $select->where('active = ?', 't');
        }
        
        $count = $this->fetchRow($select);
        
        return $count->count;
    }
        
    /**
     * Order the rows by moving them up or down, 
     * $first and $second order column values are being exchanged with each other
     * 
     * @param integer $first id
     * @param integer $second id
     * @param string $key Order column name, defaults to order 
     * @return void
     */
    public function moveItem ($first, $second, $key = 'order')
    {
        $first = $this->find($first);
        $first = $first->current();
        $firstValue = (int) $first->order;
        
        $second = $this->find($second);
        $second = $second->current();
        $secondValue = (int) $second->$key;
        
        $first->$key = $secondValue;
        $second->$key = $firstValue;
        
        $first->save();
        $second->save();
    
    }

    /**
     * Get primary key
     * 
     * @param array $options Available options, defaults to unquoted 
     * <ul><li>name bool - Return the primary key name only</li>
     * <li>quote bool - Return quoted table name and primary key name</li></ul>
     * @return string
     */
    public function getPrimaryKey (array $options = array())
    {
        if (is_array($this->_primary)) {
            $pkey = $this->_primary[1];
        } else {
            $pkey = $this->_primary;
        }
        
        if (isset($options['quote']) && $options['quote'] == true) {
            return $this->getAdapter()->quoteIdentifier($this->_name . '.' . $pkey);
        } else {
            if(isset($options['name']) && $options['name'] == true) {
                return $pkey;
            } else {
                return $this->_name . '.' . $pkey;
            }
        }
    }

}