<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Classes.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for classes table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Classes extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'classes';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get classes
     * 
     * @param $options Available options:
     * <ul><li>withClassLabels bool - If true gets classes with labels</li>
     * <li>whereStm string - Fetch rows that match the statement</li>
     * <li>whereLayerId integer - Fetch rows that match the layer id</li>
     * <li>whereId integer - Fetch rows that match the id</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'layer_id' , 
            'name' , 
            'expression' , 
            'minscaledenom' , 
            'maxscaledenom' ,
            'legend'
        );
        
        $fields = array_merge($fields, array(
            'label_active' , 
            'label_size' , 
            'label_color' , 
            'label_backgroundcolor' ,
            'label_angle' ,
            'label_buffer' , 
            'label_minfeaturesize' , 
            'label_position' ,
            'label_repeatdistance' ,
            'label_font' ,
            'label_align' ,
            'label_maxlength' ,
            'label_priority' ,
            'label_wrap'
        ));
        
        $select->from($this, $fields);
        
        if (isset($options['whereStm'])) {
            $select->where($options['whereStm']);
        }
        
        if (isset($options['whereLayerId'])) {
            $select->where($this->_name . '.layer_id = ?', $options['whereLayerId']);
        }
        
        if (isset($options['whereId'])) {
            $select->where($this->_name . '.id = ?', $options['whereId']);
        }
        
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        foreach ($result as $row) {
            //expression
            if (! empty($row->expression)) {
                if (substr($row->expression, 0, 1) != '(' && substr($row->expression, - 1) != ')') {
                    $row->expression = '(' . $row->expression . ')';
                }
            }
            
            //label_fields
            if (! empty($row->label_fields)) {
                $row->label_fields = explode(',', $row->label_fields);
            }
        }
        
        /*
        //Convert to array
        if(isset($options['asArray']) && $options['asArray'] == true) {
            $result = $result->toArray();
        }
        */
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        if (empty($data->data)) {
            $data->data = NULL;
        }
        
        if ($column == 'expression') {
            $row->$column = $data->getUnescaped('data');
        } else {
            $row->$column = $data->data;
        }
        
        return $row->save();
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        
        $layers = new Giscuit_Db_Table_Layers();
        $mapOptions = array(
            'whereId' => $data->layer_id
        );

        $rows = array_values($layers->getItems($mapOptions));
        $settings = $rows[0];
         
        $newRow = $this->createRow();
        $newRow->layer_id = $data->layer_id;
        $newRow->label_minfeaturesize = $settings['label_minfeaturesize']; 
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
        
        return $newRow;
    }

    /**
     * Add initial class for a layer
     * 
     * @param mixed $table Table settings
     * @return Zend_Db_Table_Row
     */
    public function addInitial ($table)
    {
        $camelCaseToUnderscore = new Zend_Filter_Word_CamelCaseToUnderscore();
        
        $settings = $table->getSettings();
        $layerId = $settings['id'];
        
        $newRow = $this->createRow();
        $newRow->layer_id = $layerId;
        $newRow->name = strtolower($camelCaseToUnderscore->filter($settings['class'])) . '_default';
        $newRow->label_minfeaturesize = $settings['label_minfeaturesize']; 
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
        
        return $newRow;
    }

    /**
     * Add classifying class for a layer
     * 
     * @param Zend_Filter_Input $input Filtered classes and styles data
     * @param string $expr Custom interval expression
     * @param string $intervalRange Interval range used as a class name
     * @param string $color Specified color for this class style
     * @return void
     */
    public function addClassify ($input, $expr, $intervalRange, $color)
    {
        $layers = new Giscuit_Db_Table_Layers();
        $mapOptions = array(
            'whereId' => $input->layer_id
        );

        $rows = array_values($layers->getItems($mapOptions));
        $settings = $rows[0];
         
        $newRow = $this->createRow();
        $newRow->layer_id = $input->layer_id;
        $newRow->label_minfeaturesize = $settings['label_minfeaturesize']; 
        $newRow->expression = $expr;
        $newRow->name = $intervalRange;
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
        
        $stylesTable = new Giscuit_Db_Table_Styles();
        $stylesTable->addClassify($input, $pkey, $color);
    }
    
    /**
     * Check if class belongs to current user
     * 
     * @param integer $id Class id
     * @return void
     */
    public function checkCurrUserOwnClass($id)
    {
        $rows = $this->getItems(array(
        	'whereId' => $id
        ))->toArray();
        if(count($rows) == 0) {
            throw new Exception("Invalid class id ({$id})");
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersTable->checkCurrUserOwnLayer($rows[0]['layer_id']);
    }
}
