<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: GeometryTables.php 2896 2012-08-01 15:28:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for geometry tables table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_GeometryTables extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'geometry_columns';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'f_table_name';

    /**
     * Table schema name
     * 
     * @var string
     */
    protected $_schema = 'public';
    
    /**
     * Table column types
     * 
     * @var array
     */
    public $columnTypes = array(
        'int2' => 'smallint' , 
        'int4' => 'integer' , 
        'int8' => 'bigint' , 
        'float4' => 'real' , 
        'float8' => 'double precision' , 
        'numeric' => 'numeric' , 
        'varchar' => 'character varying' , 
        'bpchar' => 'character' , 
        'text' => 'text' , 
        'bytea' => 'bytea' , 
        'timestamp' => 'timestamp without time zone' , 
        'timestamptz' => 'timestamp with time zone' , 
        'interval' => 'interval' , 
        'date' => 'date' , 
        'time' => 'time without time zone' , 
        'timetz' => 'time with time zone' , 
        'bool' => 'boolean' , 
        'geometry' => 'geometry'
    );

    /**
     * Get geometry columns
     * 
     * @param array $options Available options:
     * <ul><li>asArray bool - Get results as array</li>
     * <li>withHistorySchema bool - Also check the hisdtory schema tables</li>
     * <li>whereTables string - Get results that match this comma separated list of tables</li>
     * <li>whereName string - Get results where table name matches this parameter</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'f_table_name' , 
            'type' ,
            'f_geometry_column' , 
            'srid'
        );
        
        $select->from($this, $fields);
        
        //withHistorySchema
        if (isset($options['withHistorySchema']) && $options['withHistorySchema'] == true) {
            $select->where(
                "{$this->_name}.f_table_schema = '{$config->database->schema}_layers'" .
                "OR {$this->_name}.f_table_schema = '{$config->database->schema}_history'"
            );
        } else {
            $select->where("{$this->_name}.f_table_schema = '{$config->database->schema}_layers'");
        }
        
        //whereTables
        if (isset($options['whereTables'])) {
        	$select->where("',{$options['whereTables']}' LIKE '%,' || f_table_name || ',%'");
        }

        //whereName
        if (! empty($options['whereName'])) {
            $select->where($this->_name . '.f_table_name = ?', $options['whereName']);
        }
        
        $result = $this->fetchAll($select);
        
        //Convert to array
        if (isset($options['asArray']) && $options['asArray'] == true) {
            $result = $result->toArray();
        }
        
        return $result;
    }

    /**
     * Get all database tables
     * 
     * @param array $options Available options:
     * <ul><li>schema string - Database schema</li>
     * <li>withGeometryTables bool - Include geometry tables</li>
     * <li>whereTables string - Get results that match this comma separated list of tables</li></ul> 
     * @return array All tables
     */
    public function getAllTables (array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        if(! isset($options['schema'])) {
            $schema = $config->database->schema . '_layers';
        } else {
            $schema = $options['schema'];
        }
        
        //TODO: use PDO method to get info about postgresql tables
        $select = $this->getAdapter()->select();
        $select->from('pg_tables', array(
            'tablename'
        ));
        
        if (isset($options['withGeometryTables']) && $options['withGeometryTables'] == true) {
	        $select->joinLeft(array(
	            $this->_name
	        ), "pg_tables.schemaname = {$this->_name}.f_table_schema AND pg_tables.tablename = {$this->_name}.f_table_name", array(
	        	'type' ,
                'f_geometry_column' ,
                'srid'
	        ));
        }

        if (isset($options['whereTables'])) {
        	$select->where("'{$options['whereTables']}' LIKE '%' || tablename || ',%'");
        }
        
        $select->where('schemaname = ?', $schema);
        $select->order('tablename asc');
        $tables = $this->getAdapter()->fetchAssoc($select);

        $result = array();

        foreach ($tables as $table) {
        	$table['f_table_name'] = $table['tablename'];
            $result[$table['tablename']] = $table;
        }

        return $result;
    }

    /*
    public function getAllColumns (array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        if(! isset($options['schema'])) {
            $schema = $config->database->schema . '_layers';
        } else {
            $schema = $options['schema'];
        }
        
        $result = array();
        
        $tables = $this->getAllTables();
        
        foreach ($tables as $table) {
            $info = $this->getAdapter()->describeTable($table, $schema);
            $info = array_keys($info);
            foreach ($info as $key => $value) {
                if (isset($options['whereTable']) ) {
                    
                } else {
                    $info[$key] = $table . '.' . $value;
                }
            }
            $result = array_merge($result, $info);
        }
        
        return $result;
    
    }
    */
    
    /**
     * Get all spatial reference systems
     * 
     * @param array $options 
     * @return array
     */
    public function getAllSpatialRefSys (array $options = array())
    {

        $select = $this->getAdapter()->select();
        $select->from('public.spatial_ref_sys', array(
            'srid' ,
            'name' => new Zend_Db_Expr("regexp_matches(srtext, '[GEOGCS|PROJCS]\\\[\\\\\"(.*?)\\\\\"')")
        ));
        
        $this->getAdapter()->setFetchMode(Zend_Db::FETCH_ASSOC);
        $result = $this->getAdapter()->fetchAll($select);
        $this->getAdapter()->setFetchMode(Zend_Db::FETCH_OBJ);

        foreach($result as $key => $row) {
            $row['name'] = str_replace("{\"", NULL, $row['name']);
            $row['name'] = str_replace("{", NULL, $row['name']);
            $row['name'] = str_replace("\"}", NULL, $row['name']);
            $row['name'] = str_replace("}", NULL, $row['name']);
            
            $row['name'] = $row['name'] . " (EPSG:{$row['srid']})";
            
            $result[$key] = $row;
        }
                
        return $result;
    }
    
    /**
     * Get projection by srid
     * 
     * @param integer $srid Srid value
     * @return Zend_Db_Table_Row
     */
    public function getProjTextBySrid ($srid)
    {

        $select = $this->getAdapter()->select();
        $select->from('public.spatial_ref_sys', array(
            'proj4text' 
        ));
        $select->where($this->getAdapter()->quoteIdentifier('spatial_ref_sys.srid') . ' = ?', $srid);
        
        $result = $this->getAdapter()->fetchRow($select);
                        
        return $result;
    }
    
    /**
     * Delete specified table
     * 
     * @param string $tableName Name of the table
     * @param string $schema Database schema
     * @return void
     */
    public function deleteTable($tableName, $schema = NULL)
    {
        $config = Zend_Registry::get('config');
        
        if($schema == NULL) {
            $schema = $config->database->schema . '_layers';
        }
        
        //TODO: Check if CASCADE is SQL compliant
        $this->getAdapter()->query("DROP TABLE  IF EXISTS \"$schema\".\"$tableName\" CASCADE");
        
        //Delete record from geometry_columns table
        $this->delete("f_table_name = '$tableName' AND f_table_schema = '$schema'");
        
    }
}
